@@ -4,17 +4,19 @@ predictable NDCube objects.
 """
 import logging
 
-import astropy.nddata
-import astropy.units as u
 import dask.array
 import numpy as np
 import pytest
+
+import astropy.nddata
+import astropy.units as u
 from astropy.coordinates import SkyCoord
 from astropy.nddata import StdDevUncertainty
 from astropy.time import Time, TimeDelta
 from astropy.wcs import WCS
 
-from ndcube import ExtraCoords, GlobalCoords, NDCube, NDCubeSequence
+from ndcube import ExtraCoords, GlobalCoords, NDCube, NDCubeSequence, NDMeta
+from ndcube.tests import helpers
 
 # Force MPL to use non-gui backends for testing.
 try:
@@ -42,9 +44,9 @@ def skycoord_2d_lut(shape):
     return SkyCoord(*data, unit=u.deg)
 
 
-def data_nd(shape):
+def data_nd(shape, dtype=float):
     nelem = np.prod(shape)
-    return np.arange(nelem).reshape(shape)
+    return np.arange(nelem, dtype=dtype).reshape(shape)
 
 
 def time_extra_coords(shape, axis, base):
@@ -196,6 +198,30 @@ def wcs_3d_lt_ln_l():
     return WCS(header=header)
 
 
+@pytest.fixture
+def wcs_3d_wave_lt_ln():
+    header = {
+        'CTYPE1': 'WAVE    ',
+        'CUNIT1': 'Angstrom',
+        'CDELT1': 0.2,
+        'CRPIX1': 0,
+        'CRVAL1': 10,
+
+        'CTYPE2': 'HPLT-TAN',
+        'CUNIT2': 'deg',
+        'CDELT2': 0.5,
+        'CRPIX2': 2,
+        'CRVAL2': 0.5,
+
+        'CTYPE3': 'HPLN-TAN    ',
+        'CUNIT3': 'deg',
+        'CDELT3': 0.4,
+        'CRPIX3': 2,
+        'CRVAL3': 1,
+    }
+    return WCS(header=header)
+
+
 @pytest.fixture
 def wcs_2d_lt_ln():
     spatial = {
@@ -245,7 +271,7 @@ def wcs_3d_ln_lt_t_rotated():
         'NAXIS2': 5,
 
         'CTYPE3': 'TIME    ',
-        'CUNIT3': 'seconds',
+        'CUNIT3': 's',
         'CDELT3': 3,
         'CRPIX3': 0,
         'CRVAL3': 0,
@@ -266,6 +292,86 @@ def wcs_3d_ln_lt_t_rotated():
     return WCS(header=h_rotated)
 
 
+@pytest.fixture
+def wcs_3d_ln_lt_l_coupled():
+    # WCS for a 3D data cube with two celestial axes and one wavelength axis.
+    # The latitudinal dimension is coupled to the third pixel dimension through
+    # a single off diagonal element in the PCij matrix
+    header = {
+        'CTYPE1': 'HPLN-TAN',
+        'CRPIX1': 5,
+        'CDELT1': 5,
+        'CUNIT1': 'arcsec',
+        'CRVAL1': 0.0,
+
+        'CTYPE2': 'HPLT-TAN',
+        'CRPIX2': 5,
+        'CDELT2': 5,
+        'CUNIT2': 'arcsec',
+        'CRVAL2': 0.0,
+
+        'CTYPE3': 'WAVE',
+        'CRPIX3': 1.0,
+        'CDELT3': 1,
+        'CUNIT3': 'Angstrom',
+        'CRVAL3': 1.0,
+
+        'PC1_1': 1,
+        'PC1_2': 0,
+        'PC1_3': 0,
+        'PC2_1': 0,
+        'PC2_2': 1,
+        'PC2_3': -1.0,
+        'PC3_1': 0.0,
+        'PC3_2': 0.0,
+        'PC3_3': 1.0,
+
+        'WCSAXES': 3,
+
+        'DATEREF': "2020-01-01T00:00:00"
+    }
+    return WCS(header=header)
+
+
+@pytest.fixture
+def wcs_3d_ln_lt_t_coupled():
+    # WCS for a 3D data cube with two celestial axes and one time axis.
+    header = {
+        'CTYPE1': 'HPLN-TAN',
+        'CRPIX1': 5,
+        'CDELT1': 5,
+        'CUNIT1': 'arcsec',
+        'CRVAL1': 0.0,
+
+        'CTYPE2': 'HPLT-TAN',
+        'CRPIX2': 5,
+        'CDELT2': 5,
+        'CUNIT2': 'arcsec',
+        'CRVAL2': 0.0,
+
+        'CTYPE3': 'UTC',
+        'CRPIX3': 1.0,
+        'CDELT3': 1,
+        'CUNIT3': 's',
+        'CRVAL3': 1.0,
+
+        'PC1_1': 1,
+        'PC1_2': 0,
+        'PC1_3': 0,
+        'PC2_1': 0,
+        'PC2_2': 1,
+        'PC2_3': 0,
+        'PC3_1': 0,
+        'PC3_2': 1,
+        'PC3_3': 1,
+
+        'WCSAXES': 3,
+
+        'DATEREF': "2020-01-01T00:00:00"
+    }
+    return WCS(header=header)
+
+
 ################################################################################
 # Extra and Global Coords Fixtures
 ################################################################################
@@ -312,7 +418,6 @@ def extra_coords_sharing_axis():
                                            )
                                           )
 
-
 ################################################################################
 # NDCube Fixtures
 ################################################################################
@@ -329,10 +434,27 @@ def ndcube_4d_ln_l_t_lt(wcs_4d_lt_t_l_ln):
 def ndcube_4d_ln_lt_l_t(wcs_4d_t_l_lt_ln):
     shape = (5, 8, 10, 12)
     wcs_4d_t_l_lt_ln.array_shape = shape
-    data_cube = data_nd(shape)
+    data_cube = data_nd(shape, dtype=int)
     return NDCube(data_cube, wcs=wcs_4d_t_l_lt_ln)
 
 
+@pytest.fixture
+def ndcube_4d_axis_aware_meta(wcs_4d_t_l_lt_ln):
+    shape = (5, 8, 10, 12)
+    wcs_4d_t_l_lt_ln.array_shape = shape
+    data_cube = data_nd(shape, dtype=int)
+    meta = NDMeta({"a": "scalar",
+                   "slit position": np.arange(shape[0], dtype=int),
+                   "pixel label": np.arange(np.prod(shape[:2])).reshape(shape[:2]),
+                   "line": ["Si IV"] * shape[2],
+                   "exposure time": ([2] * shape[-1]) * u.s},
+                  axes={"slit position": 0,
+                        "pixel label": (0, 1),
+                        "line": (2,),
+                        "exposure time": 3})
+    return NDCube(data_cube, wcs=wcs_4d_t_l_lt_ln, meta=meta)
+
+
 @pytest.fixture
 def ndcube_4d_uncertainty(wcs_4d_t_l_lt_ln):
     shape = (5, 8, 10, 12)
@@ -444,6 +566,24 @@ def ndcube_3d_ln_lt_l_ec_time(wcs_3d_l_lt_ln, time_and_simple_extra_coords_2d):
     return cube
 
 
+@pytest.fixture
+def ndcube_3d_wave_lt_ln_ec_time(wcs_3d_wave_lt_ln):
+    shape = (3, 4, 5)
+    wcs_3d_wave_lt_ln.array_shape = shape
+    data = data_nd(shape)
+    mask = data > 0
+    cube = NDCube(
+        data,
+        wcs_3d_wave_lt_ln,
+        mask=mask,
+        uncertainty=data,
+    )
+    base_time = Time('2000-01-01', format='fits', scale='utc')
+    timestamps = Time([base_time + TimeDelta(60 * i, format='sec') for i in range(data.shape[0])])
+    cube.extra_coords.add('time', 0, timestamps)
+    return cube
+
+
 @pytest.fixture
 def ndcube_3d_rotated(wcs_3d_ln_lt_t_rotated, simple_extra_coords_3d):
     data_rotated = np.array([[[1, 2, 3, 4, 6], [2, 4, 5, 3, 1], [0, -1, 2, 4, 2], [3, 5, 1, 2, 0]],
@@ -459,6 +599,31 @@ def ndcube_3d_rotated(wcs_3d_ln_lt_t_rotated, simple_extra_coords_3d):
     return cube
 
 
+@pytest.fixture
+def ndcube_3d_coupled(wcs_3d_ln_lt_l_coupled):
+    shape = (128, 256, 512)
+    wcs_3d_ln_lt_l_coupled.array_shape = shape
+    data = data_nd(shape)
+    mask = data > 0
+    return NDCube(
+        data,
+        wcs_3d_ln_lt_l_coupled,
+        mask=mask,
+        uncertainty=data,
+    )
+
+
+@pytest.fixture
+def ndcube_3d_coupled_time(wcs_3d_ln_lt_t_coupled):
+    shape = (128, 256, 512)
+    wcs_3d_ln_lt_t_coupled.array_shape = shape
+    data = data_nd(shape)
+    return NDCube(
+        data,
+        wcs_3d_ln_lt_t_coupled,
+    )
+
+
 @pytest.fixture
 def ndcube_3d_l_ln_lt_ectime(wcs_3d_lt_ln_l):
     return gen_ndcube_3d_l_ln_lt_ectime(wcs_3d_lt_ln_l,
@@ -478,8 +643,7 @@ def ndcube_2d_ln_lt_uncert(wcs_2d_lt_ln):
     shape = (10, 12)
     data_cube = data_nd(shape)
     uncertainty = astropy.nddata.StdDevUncertainty(data_cube * 0.1)
-    cube = NDCube(data_cube, wcs=wcs_2d_lt_ln, uncertainty=uncertainty)
-    return cube
+    return NDCube(data_cube, wcs=wcs_2d_lt_ln, uncertainty=uncertainty)
 
 
 @pytest.fixture
@@ -492,8 +656,7 @@ def ndcube_2d_ln_lt_mask_uncert(wcs_2d_lt_ln):
     mask[2, 0] = True
     mask[3, 3] = True
     mask[4:6, :4] = True
-    cube = NDCube(data_cube, wcs=wcs_2d_lt_ln, uncertainty=uncertainty, mask=mask)
-    return cube
+    return NDCube(data_cube, wcs=wcs_2d_lt_ln, uncertainty=uncertainty, mask=mask)
 
 
 @pytest.fixture
@@ -504,7 +667,7 @@ def ndcube_2d_ln_lt_uncert_ec(wcs_2d_lt_ln):
     cube = NDCube(data_cube, wcs=wcs_2d_lt_ln, uncertainty=uncertainty)
     cube.extra_coords.add(
         "time", 0,
-        Time("2000-01-01 00:00", scale="utc") + Timedelta(np.arange(shape[0])*60, format="sec"))
+        Time("2000-01-01 00:00", scale="utc") + TimeDelta(np.arange(shape[0])*60, format="sec"))
     return cube
 
 
@@ -598,7 +761,8 @@ def ndcubesequence_4c_ln_lt_l_cax1(ndcube_3d_ln_lt_l):
     cube2.data[:] *= 2
     cube3.data[:] *= 3
     cube4.data[:] *= 4
-    return NDCubeSequence([cube1, cube2, cube3, cube4], common_axis=1)
+    meta = helpers.ndmeta_et0_pr02((4, 2, 3, 4))
+    return NDCubeSequence([cube1, cube2, cube3, cube4], common_axis=1, meta=meta)
 
 
 @pytest.fixture
