@@ -2,10 +2,12 @@ import copy
 import numbers
 import textwrap
 
-import astropy.units as u
 import numpy as np
 
+import astropy.units as u
+
 from ndcube import utils
+from ndcube.utils.exceptions import warn_deprecated
 from ndcube.visualization.descriptor import PlotterDescriptor
 
 
@@ -45,11 +47,16 @@ class NDCubeSequenceBase:
         """
         The length of each axis including the sequence axis.
         """
-        return self._dimensions
+        warn_deprecated("Replaced by ndcube.NDCubeSequence.shape")
+        return tuple([d * u.pix for d in self._shape])
 
     @property
-    def _dimensions(self):
-        dimensions = [len(self.data) * u.pix] + list(self.data[0].dimensions)
+    def shape(self):
+        return self._shape
+
+    @property
+    def _shape(self):
+        dimensions = [len(self.data), *list(self.data[0].data.shape)]
         if len(dimensions) > 1:
             # If there is a common axis, length of cube's along it may not
             # be the same. Therefore if the lengths are different,
@@ -57,10 +64,9 @@ class NDCubeSequenceBase:
             if self._common_axis is not None:
                 common_axis_lengths = [cube.data.shape[self._common_axis] for cube in self.data]
                 if len(np.unique(common_axis_lengths)) != 1:
-                    common_axis_dimensions = [cube.dimensions[self._common_axis]
-                                              for cube in self.data]
-                    dimensions[self._common_axis + 1] = u.Quantity(
-                        common_axis_dimensions, unit=common_axis_dimensions[0].unit)
+                    common_axis_dimensions = tuple([cube.shape[self._common_axis]
+                                                   for cube in self.data])
+                    dimensions[self._common_axis + 1] = common_axis_dimensions
         return tuple(dimensions)
 
     @property
@@ -68,25 +74,30 @@ class NDCubeSequenceBase:
         """
         The physical types associated with each array axis, including the sequence axis.
         """
-        return [("meta.obs.sequence",)] + self.data[0].array_axis_physical_types
+        return [("meta.obs.sequence",), *self.data[0].array_axis_physical_types]
 
     @property
     def cube_like_dimensions(self):
+        """
+        The length of each array axis as if all cubes were concatenated along the common axis.
+        """
+        warn_deprecated("Replaced by ndcube.NDCubeSequence.cube_like_shape")
+        return tuple(u.Quantity(d, unit=u.pix) for d in self.cube_like_shape)
+
+    @property
+    def cube_like_shape(self):
         """
         The length of each array axis as if all cubes were concatenated along the common axis.
         """
         if not isinstance(self._common_axis, int):
             raise TypeError("Common axis must be set.")
-        dimensions = list(self._dimensions)
-        cube_like_dimensions = list(self._dimensions[1:])
-        if dimensions[self._common_axis + 1].isscalar:
-            cube_like_dimensions[self._common_axis] = u.Quantity(
-                dimensions[0].value * dimensions[self._common_axis + 1].value, unit=u.pix)
+        dimensions = list(self.shape)
+        cube_like_shape = list(self._shape[1:])
+        if isinstance(dimensions[self._common_axis + 1], numbers.Integral):
+            cube_like_shape[self._common_axis] =  dimensions[0] * dimensions[self._common_axis + 1]
         else:
-            cube_like_dimensions[self._common_axis] = sum(dimensions[self._common_axis + 1])
-        # Combine into single Quantity
-        cube_like_dimensions = u.Quantity(cube_like_dimensions, unit=u.pix)
-        return cube_like_dimensions
+            cube_like_shape[self._common_axis] = sum(dimensions[self._common_axis + 1])
+        return cube_like_shape
 
     @property
     def cube_like_array_axis_physical_types(self):
@@ -100,8 +111,10 @@ class NDCubeSequenceBase:
     def __getitem__(self, item):
         if isinstance(item, numbers.Integral):
             return self.data[item]
+        # Determine whether meta attribute should be sliced.
+        new_meta = self.meta.slice[item] if (hasattr(self.meta, "__ndcube_can_slice__") and self.meta.__ndcube_can_slice__) else copy.deepcopy(self.meta)
         # Create an empty sequence in which to place the sliced cubes.
-        result = type(self)([], meta=self.meta, common_axis=self._common_axis)
+        result = type(self)([], meta=new_meta, common_axis=self._common_axis)
         if isinstance(item, slice):
             result.data = self.data[item]
         else:
@@ -151,8 +164,6 @@ class NDCubeSequenceBase:
         """
         common_axis = self._common_axis
         # Get coordinate objects associated with the common axis in all cubes.
-        common_axis_names = set.intersection(*[set(cube.array_axis_physical_types[common_axis])
-                                               for cube in self.data])
         common_coords = []
         mappings = []
         for i, cube in enumerate(self.data):
@@ -188,8 +199,8 @@ class NDCubeSequenceBase:
         # Collect names of global coords common to all cubes.
         global_names = set.intersection(*[set(cube.global_coords.keys()) for cube in self.data])
         # For each coord, combine values from each cube's global coords property.
-        return dict([(name, [cube.global_coords[name] for cube in self.data])
-                     for name in global_names])
+        return {name: [cube.global_coords[name] for cube in self.data]
+                     for name in global_names}
 
     def explode_along_axis(self, axis):
         """
@@ -207,7 +218,7 @@ class NDCubeSequenceBase:
         """
         # If axis is -ve then calculate the axis from the length of the dimensions of one cube.
         if axis < 0:
-            axis = len(self.dimensions[1::]) + axis
+            axis = len(self.shape[1::]) + axis
         # To store the resultant cube
         result_cubes = []
         # All slices are initially initialised as slice(None, None, None)
@@ -266,7 +277,7 @@ class NDCubeSequenceBase:
         item = self._get_sequence_crop_item(*points, wcses=wcses)
         return self[item]
 
-    def crop_by_values(self, lower_corner, upper_corner, units=None, wcses=None):
+    def crop_by_values(self, *points, units=None, wcses=None):
         """
         Crop cubes in sequence to smallest pixel-space bounding box containing the input points.
 
@@ -298,10 +309,10 @@ class NDCubeSequenceBase:
 
         Returns
         -------
-        : `~ndcube.NDCubeSequence`
+        `~ndcube.NDCubeSequence`
             The cropped sequence.
         """
-        item = self._get_sequence_crop_item(*points, wcses=wcses, crop_by_values=True, unit=units)
+        item = self._get_sequence_crop_item(*points, wcses=wcses, crop_by_values=True, units=units)
         return self[item]
 
     def _get_sequence_crop_item(self, *points, wcses=None, crop_by_values=False, units=None):
@@ -341,7 +352,7 @@ class NDCubeSequenceBase:
             Only used if crop_by_values is True.
         """
         n_cubes = len(self.data)
-        cube_ndim = len(self.dimensions[1:])
+        cube_ndim = len(self.shape[1:])
         starts = np.zeros((n_cubes, cube_ndim), dtype=int)
         stops = np.zeros((n_cubes, cube_ndim), dtype=int)
         if wcses is None:
@@ -371,12 +382,12 @@ class NDCubeSequenceBase:
         return (textwrap.dedent(f"""\
                 NDCubeSequence
                 --------------
-                Dimensions:  {self.dimensions}
+                Dimensions:  {self.shape}
                 Physical Types of Axes: {self.array_axis_physical_types}
                 Common Cube Axis: {self._common_axis}"""))
 
     def __repr__(self):
-        return f"{object.__repr__(self)}\n{str(self)}"
+        return f"{object.__repr__(self)}\n{self!s}"
 
     def __len__(self):
         return len(self.data)
@@ -466,8 +477,8 @@ class _IndexAsCubeSlicer:
 
     def __getitem__(self, item):
         common_axis = self.seq._common_axis
-        common_axis_lengths = [int(cube.dimensions[common_axis].value) for cube in self.seq.data]
-        n_cube_dims = len(self.seq.cube_like_dimensions)
+        common_axis_lengths = [int(cube.shape[common_axis]) for cube in self.seq.data]
+        n_cube_dims = len(self.seq.cube_like_shape)
         n_uncommon_cube_dims = n_cube_dims - 1
         # If item is iint or slice, turn into a tuple, filling in items
         # for unincluded axes with slice(None). This ensures it is
@@ -480,7 +491,7 @@ class _IndexAsCubeSlicer:
         # If common axis item is slice(None), result is trivial as common_axis is not changed.
         if item[common_axis] == slice(None):
             # Create item for slicing through the default API and slice.
-            return self.seq[tuple([slice(None)] + item)]
+            return self.seq[(slice(None), *item)]
         if isinstance(item[common_axis], numbers.Integral):
             # If common_axis item is an int or return an NDCube with dimensionality of N-1
             sequence_index, common_axis_index = \
@@ -490,25 +501,24 @@ class _IndexAsCubeSlicer:
             cube_item = copy.deepcopy(item)
             cube_item[common_axis] = common_axis_index
             return self.seq.data[sequence_index][tuple(cube_item)]
-        else:
-            # item can now only be a tuple whose common axis item is a non-None slice object.
-            # Convert item into iterable of SequenceItems and slice each cube appropriately.
-            # item for common_axis must always be a slice for every cube,
-            # even if it is only a length-1 slice.
-            # Thus NDCubeSequence.index_as_cube can only slice away common axis if
-            # item is int or item's first item is an int.
-            # i.e. NDCubeSequence.index_as_cube cannot cause common_axis to become None
-            # since in all cases where the common_axis is sliced away involve an NDCube
-            # is returned, not an NDCubeSequence.
-            # common_axis of returned sequence must be altered if axes in front of it
-            # are sliced away.
-            sequence_items = utils.sequence.cube_like_tuple_item_to_sequence_items(
-                item, common_axis, common_axis_lengths, n_cube_dims)
-            # Work out new common axis value if axes in front of it are sliced away.
-            new_common_axis = common_axis - sum([isinstance(i, numbers.Integral)
-                                                 for i in item[:common_axis]])
-            # Copy sequence and alter the data and common axis.
-            result = type(self.seq)([], meta=self.seq.meta, common_axis=new_common_axis)
-            result.data = [self.seq.data[sequence_item.sequence_index][sequence_item.cube_item]
-                           for sequence_item in sequence_items]
-            return result
+        # item can now only be a tuple whose common axis item is a non-None slice object.
+        # Convert item into iterable of SequenceItems and slice each cube appropriately.
+        # item for common_axis must always be a slice for every cube,
+        # even if it is only a length-1 slice.
+        # Thus NDCubeSequence.index_as_cube can only slice away common axis if
+        # item is int or item's first item is an int.
+        # i.e. NDCubeSequence.index_as_cube cannot cause common_axis to become None
+        # since in all cases where the common_axis is sliced away involve an NDCube
+        # is returned, not an NDCubeSequence.
+        # common_axis of returned sequence must be altered if axes in front of it
+        # are sliced away.
+        sequence_items = utils.sequence.cube_like_tuple_item_to_sequence_items(
+            item, common_axis, common_axis_lengths, n_cube_dims)
+        # Work out new common axis value if axes in front of it are sliced away.
+        new_common_axis = common_axis - sum([isinstance(i, numbers.Integral)
+                                             for i in item[:common_axis]])
+        # Copy sequence and alter the data and common axis.
+        result = type(self.seq)([], meta=self.seq.meta, common_axis=new_common_axis)
+        result.data = [self.seq.data[sequence_item.sequence_index][sequence_item.cube_item]
+                       for sequence_item in sequence_items]
+        return result
