@@ -1,11 +1,13 @@
 import warnings
 
-import astropy.units as u
 import matplotlib.pyplot as plt
 import numpy as np
+
+import astropy.units as u
 from astropy.utils.exceptions import AstropyUserWarning
 from astropy.visualization.wcsaxes import WCSAxes
 
+from ndcube.utils.exceptions import warn_user
 from . import plotting_utils as utils
 from .base import BasePlotter
 from .descriptor import MISSING_ANIMATORS_ERROR_MSG
@@ -72,7 +74,7 @@ class MatplotlibPlotter(BasePlotter):
 
         # Check kwargs are in consistent formats and set default values if not done so by user.
         plot_axes, axes_coordinates, axes_units = utils.prep_plot_kwargs(
-            len(self._ndcube.dimensions), plot_wcs, plot_axes, axes_coordinates, axes_units)
+            len(self._ndcube.shape), plot_wcs, plot_axes, axes_coordinates, axes_units)
 
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', AstropyUserWarning)
@@ -94,7 +96,7 @@ class MatplotlibPlotter(BasePlotter):
         """
         Based on an axes object and axes_coords, work out which coords should not be visible.
         """
-        visible_coords = set(item[1] for item in axes.coords._aliases.items() if item[0] in axes_coordinates)
+        visible_coords = {item[1] for item in axes.coords._aliases.items() if item[0] in axes_coordinates}
         return set(axes.coords._aliases.values()).difference(visible_coords)
 
     def _apply_axes_coordinates(self, axes, axes_coordinates):
@@ -229,7 +231,7 @@ class MatplotlibPlotter(BasePlotter):
         axes. See https://wcsaxes.readthedocs.io for more information.
         """
         kwargs = {'wcs': self._ndcube.wcs}
-        n_dim = len(self._ndcube.dimensions)
+        n_dim = len(self._ndcube.shape)
         if n_dim > 2:
             kwargs['slices'] = ['x', 'y'] + [None] * (n_dim - 2)
         return WCSAxes, kwargs
@@ -252,10 +254,9 @@ class MatplotlibPlotter(BasePlotter):
 
         # TODO: Add support for transposing the array.
         if 'y' in plot_axes and plot_axes.index('y') < plot_axes.index('x'):
-            warnings.warn(
+            warn_user(
                 "Animating a NDCube does not support transposing the array. The world axes "
-                "may not display as expected because the array will not be transposed.",
-                UserWarning
+                "may not display as expected because the array will not be transposed."
             )
         plot_axes = [p if p is not None else 0 for p in plot_axes]
 
