@@ -1,8 +1,9 @@
-import astropy.units as u
 import matplotlib.pyplot as plt
 import mpl_animators
 import numpy as np
 import pytest
+
+import astropy.units as u
 from astropy.visualization.wcsaxes import WCSAxes
 from astropy.wcs import WCS
 
@@ -21,7 +22,7 @@ def test_plot_1D_cube(ndcube_1d_l):
 
 @figure_test
 @pytest.mark.parametrize(("ndcube_4d", "cslice", "kwargs"),
-                         (
+                         [
                              ("ln_lt_l_t", np.s_[0, 0, 0, :], {}),
                              ("ln_lt_l_t", np.s_[0, 0, :, 0], {}),
                              ("ln_lt_l_t", np.s_[0, :, 0, 0], {}),
@@ -30,7 +31,7 @@ def test_plot_1D_cube(ndcube_1d_l):
                              ("uncertainty", np.s_[0, 0, 0, :], {}),
                              ("unit_uncertainty", np.s_[0, 0, 0, :], {'data_unit': u.mJ}),
 
-                             ("mask", np.s_[0, 0, 0, :], {'marker': 'o'}),),
+                             ("mask", np.s_[0, 0, 0, :], {'marker': 'o'}),],
                          indirect=["ndcube_4d"])
 def test_plot_1D_cube_from_slice(ndcube_4d, cslice, kwargs):
     # TODO: The output for the spatial plots is inconsistent between the lat
@@ -79,12 +80,12 @@ def test_plot_2D_cube_custom_axis_plot_axes(ndcube_2d_ln_lt):
 
 @figure_test
 @pytest.mark.parametrize(("ndcube_4d", "cslice", "kwargs"),
-                         (
+                         [
                              ("ln_lt_l_t", np.s_[0, 0, :, :], {}),
                              ("ln_lt_l_t", np.s_[0, :, :, 0], {}),
                              ("ln_lt_l_t", np.s_[:, :, 0, 0], {}),
                              ("unit_uncertainty", np.s_[0, 0, :, :], {'data_unit': u.mJ}),
-                             ("mask", np.s_[0, :, 0, :], {}),),
+                             ("mask", np.s_[0, :, 0, :], {}),],
                          indirect=["ndcube_4d"])
 def test_plot_2D_cube_from_slice(ndcube_4d, cslice, kwargs):
     fig = plt.figure()
@@ -107,16 +108,17 @@ def test_animate_2D_cube(ndcube_2d_ln_lt):
 
 @figure_test
 @pytest.mark.parametrize(("ndcube_4d", "cslice", "kwargs"),
-                         (
+                         [
                              ("ln_lt_l_t", np.s_[:, :, 0, :], {}),
                              ("ln_lt_l_t", np.s_[:, :, 0, :], {'plot_axes': [..., 'x']}),
                              ("ln_lt_l_t", None, {}),
-                             ("ln_lt_l_t", None, {"plot_axes": [0, 0, 'x', 'y'], "axes_units": [None, None, u.pm, None]}),
+                             ("ln_lt_l_t", None, {"plot_axes": [0, 0, 'x', 'y'],
+                                                  "axes_units": [None, None, u.pm, None]}),
                              ("ln_lt_l_t", None, {"plot_axes": [0, 'x', 0, 'y']}),
                              ("ln_lt_l_t", np.s_[0, :, :, :], {}),
                              ("ln_lt_l_t", np.s_[:, :, :, :], {}),
                              ("unit_uncertainty", np.s_[0, :, :, :], {'data_unit': u.mJ}),
-                             ("mask", np.s_[:, :, :, :], {}),),
+                             ("mask", np.s_[:, :, :, :], {}),],
                          indirect=["ndcube_4d"])
 def test_animate_cube_from_slice(ndcube_4d, cslice, kwargs):
     if cslice:
@@ -138,7 +140,7 @@ def test_mpl_axes(ndcube_4d, cslice):
     plt.close()
 
 
-def test_plotter_is_None(ndcube_1d_l):
+def test_plotter_is_none(ndcube_1d_l):
     class NewCube(NDCube):
         plotter = PlotterDescriptor(default_type=None)
 
